==> nono-0.7.2.tar.gz: nono-0.7.2/vm/qemusysctlr.cpp <==
//
// nono
// Copyright (C) 2024 nono project
// Licensed under nono-license.txt
//

//
// QEMU Virtual System Controller
//

// https://www.qemu.org/docs/master/specs/virt-ctlr.html

#include "qemusysctlr.h"
#include "power.h"

// コンストラクタ
QemuSysCtlrDevice::QemuSysCtlrDevice()
	: inherited(OBJ_QEMUSYSCTLR)
{
}

// デストラクタ
QemuSysCtlrDevice::~QemuSysCtlrDevice()
{
}

busdata
QemuSysCtlrDevice::Read32(uint32 addr)
{
	uint32 offset = addr & 0xff;

	if (offset == 0) {
		// Features (Read Only)
		uint32 data = FEATURES;
		putlog(2, "FEATURES -> $%08x", data);
		return data;
	} else {
		// 他のポートがどうなるのかは仕様に書いてない。
		putlog(1, "Read32 $%08x (BusErr)", addr);
		return busdata::BusErr;
	}
}

busdata
QemuSysCtlrDevice::Write32(uint32 addr, uint32 data)
{
	uint32 offset = addr & 0xff;

	if (offset == 4) {
		// Command (Write Only)
		static const char *cmdname[] = {
			"no-op",
			"reset",
			"halt",
			"panic",
		};
		putlog(1, "Command <- $%08x (%s)", data,
			(data < countof(cmdname) ? cmdname[data] : cmdname[0]));
		switch (data) {
		 default:
		 case 0:	// no-op
			break;

		 case 1:	// reset
			GetPowerDevice()->MakeResetHard();
			break;

		 case 2:	// halt
		 case 3:	// panic
			GetPowerDevice()->PushPowerButton();
			break;
		}
		return 0;
	} else {
		// 他のポートがどうなるのかは仕様に書いてない。
		putlog(1, "Write32 $%08x <- $%08x (BusErr)", addr, data);
		return busdata::BusErr;
	}
}

busdata
QemuSysCtlrDevice::Peek8(uint32 addr)
{
	uint32 offset = addr & 0xff;

	switch (offset) {
	 case 0:	return (FEATURES >> 24);
	 case 1:	return (FEATURES >> 16) & 0xff;
	 case 2:	return (FEATURES >>  8) & 0xff;
	 case 3:	return (FEATURES      ) & 0xff;

	 default:
		return busdata::BusErr;
	}
}

==> nono-1.1.1.tar.gz: nono-1.1.1/vm/qemusysctlr.cpp <==
//
// nono
// Copyright (C) 2024 nono project
// Licensed under nono-license.txt
//

//
// QEMU Virtual System Controller
//

// https://www.qemu.org/docs/master/specs/virt-ctlr.html

#include "qemusysctlr.h"
#include "mpu.h"
#include "power.h"

// コンストラクタ
QemuSysCtlrDevice::QemuSysCtlrDevice()
	: inherited(OBJ_QEMUSYSCTLR)
{
}

// デストラクタ
QemuSysCtlrDevice::~QemuSysCtlrDevice()
{
}

busdata
QemuSysCtlrDevice::ReadPort(uint32 offset)
{
	busdata data;

	if (offset == 0) {
		// Features (Read Only)
		data = FEATURES;
		putlog(2, "FEATURES -> $%08x", data.Data());
	} else {
		// 他のポートがどうなるのかは仕様に書いてない。
		data.SetBusErr();
		putlog(1, "Read $%08x (BusErr)", mpu->GetPaddr());
	}

	data |= BusData::Size4;
	return data;
}

busdata
QemuSysCtlrDevice::WritePort(uint32 offset, uint32 data)
{
	busdata r;

	if (offset == 1) {
		// Command (Write Only)
		static const char *cmdname[] = {
			"no-op",
			"reset",
			"halt",
			"panic",
		};
		putlog(1, "Command <- $%08x (%s)", data,
			(data < countof(cmdname) ? cmdname[data] : cmdname[0]));
		switch (data) {
		 default:
		 case 0:	// no-op
			break;

		 case 1:	// reset
			GetPowerDevice()->MakeResetHard();
			break;

		 case 2:	// halt
		 case 3:	// panic
			GetPowerDevice()->PushPowerButton();
			break;
		}
	} else {
		// 他のポートがどうなるのかは仕様に書いてない。
		putlog(1, "Write $%08x <- $%08x (BusErr)", mpu->GetPaddr(), data);
		r.SetBusErr();
	}

	r |= BusData::Size4;
	return r;
}

busdata
QemuSysCtlrDevice::PeekPort(uint32 offset)
{
	switch (offset) {
	 case 0:	return FEATURES;
	 default:
		return BusData::BusErr;
	}
}
